@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,6 +35,7 @@
 #include "photos-base-item.h"
 #include "photos-collection-icon-watcher.h"
 #include "photos-delete-item-job.h"
+#include "photos-icons.h"
 #include "photos-print-operation.h"
 #include "photos-query.h"
 #include "photos-selection-controller.h"
@@ -135,7 +136,7 @@ photos_base_item_check_effects_and_update_info (PhotosBaseItem *self)
 
   if (priv->favorite)
     {
-      pix = photos_base_item_create_symbolic_emblem ("emblem-favorite");
+      pix = photos_base_item_create_symbolic_emblem (PHOTOS_ICON_FAVORITE);
       emblem_icons = g_list_prepend (emblem_icons, pix);
     }
 
@@ -607,7 +608,7 @@ photos_base_item_update_icon_from_type (PhotosBaseItem *self)
 {
   PhotosBaseItemPrivate *priv = self->priv;
   GIcon *icon = NULL;
-  GtkIconInfo *info;
+  GtkIconInfo *info = NULL;
   GtkIconTheme *theme;
   gint icon_size;
 
@@ -630,6 +631,9 @@ photos_base_item_update_icon_from_type (PhotosBaseItem *self)
     }
 
   photos_base_item_check_effects_and_update_info (self);
+
+  g_clear_object (&info);
+  g_clear_object (&icon);
 }
 
 
