@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,6 +35,13 @@ struct _PhotosBaseManagerPrivate
 {
   GHashTable *objects;
   GObject *active_object;
+  gchar *title;
+};
+
+enum
+{
+  PROP_0,
+  PROP_TITLE
 };
 
 enum
@@ -52,6 +59,20 @@ static guint signals[LAST_SIGNAL] = { 0 };
 G_DEFINE_TYPE_WITH_PRIVATE (PhotosBaseManager, photos_base_manager, G_TYPE_OBJECT);
 
 
+static gchar *
+photos_base_manager_default_get_filter (PhotosBaseManager *self, gint flags)
+{
+  return g_strdup ("(true)");
+}
+
+
+static gchar *
+photos_base_manager_default_get_where (PhotosBaseManager *self, gint flags)
+{
+  return g_strdup ("");
+}
+
+
 static gboolean
 photos_base_manager_default_set_active_object (PhotosBaseManager *self, GObject *object)
 {
@@ -71,59 +92,6 @@ photos_base_manager_default_set_active_object (PhotosBaseManager *self, GObject
 }
 
 
-static gchar *
-photos_base_manager_get_all_filter (PhotosBaseManager *self)
-{
-  GList *l;
-  GList *values;
-  const gchar *blank = "(true)";
-  gchar *filter;
-  gchar **strv;
-  gchar *tmp;
-  guint i;
-  guint length;
-
-  values = g_hash_table_get_values (self->priv->objects);
-  length = g_list_length (values);
-  strv = (gchar **) g_malloc0_n (length + 1, sizeof (gchar *));
-
-  for (i = 0, l = values; l != NULL; l = l->next)
-    {
-      gchar *id;
-
-      g_object_get (l->data, "id", &id, NULL);
-      if (g_strcmp0 (id, "all") != 0)
-        {
-          gchar *str;
-
-          str = photos_filterable_get_filter (PHOTOS_FILTERABLE (l->data));
-          if (g_strcmp0 (str, blank) == 0)
-            g_free (str);
-          else
-            {
-              strv[i] = str;
-              i++;
-            }
-        }
-      g_free (id);
-    }
-
-  length = g_strv_length (strv);
-  if (length == 0)
-    strv[0] = g_strdup (blank);
-
-  filter = g_strjoinv (" || ", strv);
-  g_strfreev (strv);
-
-  tmp = filter;
-  filter = g_strconcat ("(", filter, ")", NULL);
-  g_free (tmp);
-
-  g_list_free (values);
-  return filter;
-}
-
-
 static void
 photos_base_manager_dispose (GObject *object)
 {
@@ -142,6 +110,35 @@ photos_base_manager_dispose (GObject *object)
 }
 
 
+static void
+photos_base_manager_finalize (GObject *object)
+{
+  PhotosBaseManager *self = PHOTOS_BASE_MANAGER (object);
+
+  g_free (self->priv->title);
+
+  G_OBJECT_CLASS (photos_base_manager_parent_class)->finalize (object);
+}
+
+
+static void
+photos_base_manager_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+  PhotosBaseManager *self = PHOTOS_BASE_MANAGER (object);
+
+  switch (prop_id)
+    {
+    case PROP_TITLE:
+      self->priv->title = g_value_dup_string (value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
 static void
 photos_base_manager_init (PhotosBaseManager *self)
 {
@@ -160,8 +157,20 @@ photos_base_manager_class_init (PhotosBaseManagerClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
   object_class->dispose = photos_base_manager_dispose;
+  object_class->finalize = photos_base_manager_finalize;
+  object_class->set_property = photos_base_manager_set_property;
+  class->get_filter = photos_base_manager_default_get_filter;
+  class->get_where = photos_base_manager_default_get_where;
   class->set_active_object = photos_base_manager_default_set_active_object;
 
+  g_object_class_install_property (object_class,
+                                   PROP_TITLE,
+                                   g_param_spec_string ("title",
+                                                        "Title",
+                                                        "The name of this manager",
+                                                        NULL,
+                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
+
   signals[ACTIVE_CHANGED] = g_signal_new ("active-changed",
                                           G_TYPE_FROM_CLASS (class),
                                           G_SIGNAL_RUN_LAST,
@@ -249,29 +258,65 @@ photos_base_manager_get_active_object (PhotosBaseManager *self)
 
 
 gchar *
-photos_base_manager_get_filter (PhotosBaseManager *self)
+photos_base_manager_get_all_filter (PhotosBaseManager *self)
 {
-  PhotosBaseManagerPrivate *priv = self->priv;
+  GList *l;
+  GList *values;
   const gchar *blank = "(true)";
   gchar *filter;
-  gchar *id;
+  gchar **strv;
+  gchar *tmp;
+  guint i;
+  guint length;
 
-  if (priv->active_object == NULL)
-    return g_strdup (blank);
+  values = g_hash_table_get_values (self->priv->objects);
+  length = g_list_length (values);
+  strv = (gchar **) g_malloc0_n (length + 1, sizeof (gchar *));
 
-  g_return_val_if_fail (PHOTOS_IS_FILTERABLE (priv->active_object), g_strdup (blank));
+  for (i = 0, l = values; l != NULL; l = l->next)
+    {
+      gchar *id;
 
-  g_object_get (priv->active_object, "id", &id, NULL);
-  if (g_strcmp0 (id, "all") == 0)
-    filter = photos_base_manager_get_all_filter (self);
-  else
-    filter = photos_filterable_get_filter (PHOTOS_FILTERABLE (priv->active_object));
+      g_object_get (l->data, "id", &id, NULL);
+      if (g_strcmp0 (id, "all") != 0)
+        {
+          gchar *str;
 
-  g_free (id);
+          str = photos_filterable_get_filter (PHOTOS_FILTERABLE (l->data));
+          if (g_strcmp0 (str, blank) == 0)
+            g_free (str);
+          else
+            {
+              strv[i] = str;
+              i++;
+            }
+        }
+      g_free (id);
+    }
+
+  length = g_strv_length (strv);
+  if (length == 0)
+    strv[0] = g_strdup (blank);
+
+  filter = g_strjoinv (" || ", strv);
+  g_strfreev (strv);
+
+  tmp = filter;
+  filter = g_strconcat ("(", filter, ")", NULL);
+  g_free (tmp);
+
+  g_list_free (values);
   return filter;
 }
 
 
+gchar *
+photos_base_manager_get_filter (PhotosBaseManager *self, gint flags)
+{
+  return PHOTOS_BASE_MANAGER_GET_CLASS (self)->get_filter (self, flags);
+}
+
+
 GObject *
 photos_base_manager_get_object_by_id (PhotosBaseManager *self, const gchar *id)
 {
@@ -299,6 +344,20 @@ photos_base_manager_get_objects_count (PhotosBaseManager *self)
 }
 
 
+const gchar *
+photos_base_manager_get_title (PhotosBaseManager *self)
+{
+  return self->priv->title;
+}
+
+
+gchar *
+photos_base_manager_get_where (PhotosBaseManager *self, gint flags)
+{
+  return PHOTOS_BASE_MANAGER_GET_CLASS (self)->get_where (self, flags);
+}
+
+
 void
 photos_base_manager_process_new_objects (PhotosBaseManager *self, GHashTable *new_objects)
 {
