@@ -528,9 +528,10 @@ photos_dlna_renderer_unshare_remove_file_cb (GObject *source_object,
   dleyna_push_host_call_remove_file_finish (DLEYNA_PUSH_HOST (source_object), res, &error);
   if (error != NULL)
     {
-      /* Assume that ignoring RemoveFile() errors is safe, since they are
-       * likely caused by the file being already removed or the DBus service
-       * having been restarted.  */
+      /* Assume that ignoring RemoveFile() errors is safe, since they
+       * are likely caused by the file being already removed or the
+       * DBus service having been restarted.
+       */
       g_warning ("Failed to call the RemoveFile method: %s", error->message);
       g_error_free (error);
       success = FALSE;
@@ -666,29 +667,42 @@ photos_dlna_renderer_device_get_icon_cb (GObject *source_object,
   GTask *task = G_TASK (user_data);
   GInputStream *icon_stream;
   GdkPixbuf *pixbuf;
+  GtkIconSize size;
   GVariant *icon_variant;
   GBytes *icon_bytes;
   const gchar *icon_data;
+  gint height = -1;
+  gint width = -1;
   gsize icon_data_size;
   GError *error = NULL;
 
-  /* The icon data is forced to be a GVariant since the GDBus bindings assume
-   * bytestrings (type 'ay') to be nul-terminated and thus do not return the length
-   * of the buffer */
+  /* The icon data is forced to be a GVariant since the GDBus bindings
+   * assume bytestrings (type 'ay') to be nul-terminated and thus do
+   * not return the length of the buffer.
+   */
   dleyna_renderer_device_call_get_icon_finish (DLEYNA_RENDERER_DEVICE (source_object),
                                                &icon_variant, NULL, res, &error);
   RETURN_ON_ERROR (task, error, "Failed to call the GetIcon method");
 
-  /* We know that the serialization of variant containing just a byte array
-   * 'ay' is the byte  array itself */
+  /* We know that the serialization of variant containing just a byte
+   * array 'ay' is the byte array itself.
+   */
   icon_bytes = g_variant_get_data_as_bytes (icon_variant);
   g_variant_unref (icon_variant);
 
   icon_data = g_bytes_get_data (icon_bytes, &icon_data_size);
   g_bytes_unref (icon_bytes);
 
+  size = (GtkIconSize) GPOINTER_TO_INT (g_task_get_task_data (task));
+  gtk_icon_size_lookup (size, &width, &height);
+
   icon_stream = g_memory_input_stream_new_from_data (icon_data, icon_data_size, NULL);
-  pixbuf = gdk_pixbuf_new_from_stream (icon_stream, g_task_get_cancellable (task), &error);
+  pixbuf = gdk_pixbuf_new_from_stream_at_scale (icon_stream,
+                                                width,
+                                                height,
+                                                TRUE,
+                                                g_task_get_cancellable (task),
+                                                &error);
   g_object_unref (icon_stream);
 
   RETURN_ON_ERROR (task, error, "Failed to parse icon data");
@@ -719,8 +733,9 @@ photos_dlna_renderer_get_udn (PhotosDlnaRenderer *self)
 
 void
 photos_dlna_renderer_get_icon (PhotosDlnaRenderer *self,
-                               const gchar *resolution,
                                const gchar *requested_mimetype,
+                               const gchar *resolution,
+                               GtkIconSize size,
                                GCancellable *cancellable,
                                GAsyncReadyCallback callback,
                                gpointer user_data)
@@ -729,6 +744,7 @@ photos_dlna_renderer_get_icon (PhotosDlnaRenderer *self,
   GTask *task;
 
   task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_task_data (task, GINT_TO_POINTER (size), NULL);
 
   dleyna_renderer_device_call_get_icon (priv->device, requested_mimetype, resolution,
                                         cancellable, photos_dlna_renderer_device_get_icon_cb,
