@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,44 +30,60 @@
 #include <glib/gi18n.h>
 
 #include "photos-application.h"
+#include "photos-collection-manager.h"
 #include "photos-embed.h"
-#include "photos-empty-results-box.h"
-#include "photos-error-box.h"
 #include "photos-indexing-notification.h"
 #include "photos-item-manager.h"
-#include "photos-main-toolbar.h"
 #include "photos-mode-controller.h"
 #include "photos-notification-manager.h"
-#include "photos-offset-overview-controller.h"
 #include "photos-preview-view.h"
+#include "photos-search-controller.h"
 #include "photos-selection-toolbar.h"
 #include "photos-spinner-box.h"
-#include "photos-tracker-change-monitor.h"
+#include "photos-search-type.h"
+#include "photos-search-type-manager.h"
+#include "photos-source.h"
+#include "photos-source-manager.h"
 #include "photos-tracker-overview-controller.h"
 #include "photos-view-container.h"
 
 
+typedef struct _PhotosEmbedSearchState PhotosEmbedSearchState;
+
+struct _PhotosEmbedSearchState
+{
+  GObject *search_type;
+  GObject *source;
+  gboolean saved;
+  gchar *str;
+};
+
 struct _PhotosEmbedPrivate
 {
+  GAction *search_action;
   GCancellable *loader_cancellable;
   GtkWidget *collections;
-  GtkWidget *error_box;
   GtkWidget *favorites;
   GtkWidget *indexing_ntfctn;
   GtkWidget *no_results;
   GtkWidget *ntfctn_mngr;
   GtkWidget *overview;
   GtkWidget *preview;
+  GtkWidget *search;
   GtkWidget *selection_toolbar;
   GtkWidget *spinner_box;
   GtkWidget *stack;
   GtkWidget *stack_overlay;
   GtkWidget *toolbar;
+  PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
+  PhotosBaseManager *src_mngr;
+  PhotosBaseManager *srch_mngr;
+  PhotosEmbedSearchState search_state;
   PhotosModeController *mode_cntrlr;
-  PhotosOffsetController *offset_cntrlr;
-  PhotosTrackerChangeMonitor *monitor;
+  PhotosSearchController *srch_cntrlr;
   PhotosTrackerController *trk_ovrvw_cntrlr;
+  PhotosWindowMode old_mode;
   guint load_show_id;
 };
 
@@ -75,7 +91,29 @@ struct _PhotosEmbedPrivate
 G_DEFINE_TYPE_WITH_PRIVATE (PhotosEmbed, photos_embed, GTK_TYPE_BOX);
 
 
-static void photos_embed_changes_pending (PhotosEmbed *self, GHashTable *changes);
+static void photos_embed_search_changed (PhotosEmbed *self);
+
+
+static void
+photos_embed_block_search_changed (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  g_signal_handlers_block_by_func (priv->src_mngr, photos_embed_search_changed, self);
+  g_signal_handlers_block_by_func (priv->srch_mngr, photos_embed_search_changed, self);
+  g_signal_handlers_block_by_func (priv->srch_cntrlr, photos_embed_search_changed, self);
+}
+
+
+static void
+photos_embed_unblock_search_changed (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  g_signal_handlers_unblock_by_func (priv->src_mngr, photos_embed_search_changed, self);
+  g_signal_handlers_unblock_by_func (priv->srch_mngr, photos_embed_search_changed, self);
+  g_signal_handlers_unblock_by_func (priv->srch_cntrlr, photos_embed_search_changed, self);
+}
 
 
 static void
@@ -91,6 +129,17 @@ photos_embed_clear_load_timer (PhotosEmbed *self)
 }
 
 
+static void
+photos_embed_clear_search (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  g_clear_object (&priv->search_state.search_type);
+  g_clear_object (&priv->search_state.source);
+  g_clear_pointer (&priv->search_state.str, g_free);
+}
+
+
 static void
 photos_embed_prepare_for_preview (PhotosEmbed *self)
 {
@@ -143,6 +192,10 @@ photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user
       view_container = NULL;
       break;
 
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      view_container = priv->search;
+      break;
+
     default:
       g_assert_not_reached ();
     }
@@ -192,11 +245,68 @@ photos_embed_load_show_timeout (gpointer user_data)
 }
 
 
+static void
+photos_embed_restore_search (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  if (!priv->search_state.saved)
+    return;
+
+  photos_base_manager_set_active_object (priv->src_mngr, priv->search_state.source);
+  photos_base_manager_set_active_object (priv->srch_mngr, priv->search_state.search_type);
+  photos_search_controller_set_string (priv->srch_cntrlr, priv->search_state.str);
+  priv->search_state.saved = FALSE;
+
+  photos_embed_clear_search (self);
+}
+
+
+static void
+photos_embed_save_search (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  if (priv->search_state.saved)
+    return;
+
+  photos_embed_clear_search (self);
+
+  priv->search_state.source = g_object_ref (photos_base_manager_get_active_object (priv->src_mngr));
+  priv->search_state.search_type = g_object_ref (photos_base_manager_get_active_object (priv->srch_mngr));
+  priv->search_state.str = g_strdup (photos_search_controller_get_string (priv->srch_cntrlr));
+  priv->search_state.saved = TRUE;
+}
+
+
 static void
 photos_embed_active_changed (PhotosBaseManager *manager, GObject *object, gpointer user_data)
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
   PhotosEmbedPrivate *priv = self->priv;
+  PhotosWindowMode mode;
+  GObject *active_collection;
+  GObject *active_item;
+  GVariant *state;
+  gboolean show_search;
+
+  /* Hide the search bar when we are moving from the search to the
+   * preview or collection viewin. Restore it when we are back.
+   */
+
+  active_collection = photos_base_manager_get_active_object (priv->col_mngr);
+  active_item = photos_base_manager_get_active_object (priv->item_mngr);
+  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  show_search = (mode == PHOTOS_WINDOW_MODE_PREVIEW && active_item == NULL && active_collection == NULL)
+                || (mode == PHOTOS_WINDOW_MODE_SEARCH && active_item == NULL);
+
+  if (show_search)
+    photos_embed_restore_search (self);
+  else
+    photos_embed_save_search (self);
+
+  state = g_variant_new ("b", show_search);
+  g_action_change_state (priv->search_action, state);
 
   if (object == NULL)
     return;
@@ -242,6 +352,10 @@ photos_embed_restore_last_page (PhotosEmbed *self)
       page = "preview";
       break;
 
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      page = "search";
+      break;
+
     default:
       g_assert_not_reached ();
       break;
@@ -251,50 +365,6 @@ photos_embed_restore_last_page (PhotosEmbed *self)
 }
 
 
-static void
-photos_embed_hide_no_results_page (PhotosEmbed *self)
-{
-  PhotosEmbedPrivate *priv = self->priv;
-
-  if (G_LIKELY (priv->monitor != NULL))
-    g_signal_handlers_disconnect_by_func (priv->monitor, photos_embed_changes_pending, self);
-
-  photos_embed_restore_last_page (self);
-}
-
-
-static void
-photos_embed_changes_pending (PhotosEmbed *self, GHashTable *changes)
-{
-  photos_embed_hide_no_results_page (self);
-}
-
-
-static void
-photos_embed_count_changed (PhotosEmbed *self, gint count)
-{
-  PhotosEmbedPrivate *priv = self->priv;
-  PhotosWindowMode mode;
-
-  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
-  if (mode != PHOTOS_WINDOW_MODE_OVERVIEW)
-    return;
-
-  if (count == 0)
-    {
-      if (G_LIKELY (priv->monitor != NULL))
-        g_signal_connect_object (priv->monitor,
-                                 "changes-pending",
-                                 G_CALLBACK (photos_embed_changes_pending),
-                                 self,
-                                 G_CONNECT_SWAPPED);
-      gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "no-results");
-    }
-  else
-    photos_embed_hide_no_results_page (self);
-}
-
-
 static void
 photos_embed_fullscreen_changed (PhotosModeController *mode_cntrlr, gboolean fullscreen, gpointer user_data)
 {
@@ -306,14 +376,29 @@ photos_embed_notify_visible_child (PhotosEmbed *self)
 {
   PhotosEmbedPrivate *priv = self->priv;
   GtkWidget *visible_child;
+  GVariant *state;
+  PhotosWindowMode mode = PHOTOS_WINDOW_MODE_NONE;
 
   visible_child = gtk_stack_get_visible_child (GTK_STACK (priv->stack));
   if (visible_child == priv->overview)
-    photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
+    mode = PHOTOS_WINDOW_MODE_OVERVIEW;
   else if (visible_child == priv->collections)
-    photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_COLLECTIONS);
+    mode = PHOTOS_WINDOW_MODE_COLLECTIONS;
   else if (visible_child == priv->favorites)
-    photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_FAVORITES);
+    mode = PHOTOS_WINDOW_MODE_FAVORITES;
+
+  if (mode == PHOTOS_WINDOW_MODE_NONE)
+    return;
+
+  if (!photos_main_toolbar_is_focus (PHOTOS_MAIN_TOOLBAR (priv->toolbar)))
+    {
+      photos_embed_block_search_changed (self);
+      state = g_variant_new ("b", FALSE);
+      g_action_change_state (priv->search_action, state);
+      photos_embed_unblock_search_changed (self);
+    }
+
+  photos_mode_controller_set_window_mode (priv->mode_cntrlr, mode);
 }
 
 
@@ -372,19 +457,20 @@ photos_embed_prepare_for_overview (PhotosEmbed *self)
 
 
 static void
-photos_embed_set_error (PhotosEmbed *self, const gchar *primary, const gchar *secondary)
+photos_embed_prepare_for_search (PhotosEmbed *self)
 {
   PhotosEmbedPrivate *priv = self->priv;
 
-  photos_error_box_update (PHOTOS_ERROR_BOX (priv->error_box), primary, secondary);
-  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "error");
-}
+  photos_base_manager_set_active_object (priv->item_mngr, NULL);
 
+  if (priv->loader_cancellable != NULL)
+    {
+      g_cancellable_cancel (priv->loader_cancellable);
+      g_clear_object (&priv->loader_cancellable);
+    }
 
-static void
-photos_embed_query_error (PhotosEmbed *self, const gchar *primary, const gchar *secondary)
-{
-  photos_embed_set_error (self, primary, secondary);
+  photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
+  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "search");
 }
 
 
@@ -406,6 +492,58 @@ photos_embed_query_status_changed (PhotosEmbed *self, gboolean querying)
 }
 
 
+static void
+photos_embed_search_changed (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+  GObject *object;
+  PhotosWindowMode mode;
+  const gchar *str;
+  gchar *search_type_id;
+  gchar *source_id;
+
+  /* Whenever a search constraint is specified we want to switch to
+   * the search mode, and when all constraints have been lifted we
+   * want to go back to the previous mode which can be either
+   * collections, favorites or overview.
+   *
+   * However there are some exceptions, which are taken care of
+   * elsewhere:
+   *  - when moving from search to preview or collection view
+   *  - when in preview
+   */
+  object = photos_base_manager_get_active_object (priv->item_mngr);
+  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  if (mode == PHOTOS_WINDOW_MODE_SEARCH && object != NULL)
+    return;
+  if (mode == PHOTOS_WINDOW_MODE_PREVIEW)
+    return;
+
+  if (priv->old_mode == PHOTOS_WINDOW_MODE_PREVIEW)
+    return;
+
+  object = photos_base_manager_get_active_object (priv->src_mngr);
+  g_object_get (object, "id", &source_id, NULL);
+
+  object = photos_base_manager_get_active_object (priv->srch_mngr);
+  g_object_get (object, "id", &search_type_id, NULL);
+
+  str = photos_search_controller_get_string (priv->srch_cntrlr);
+
+  if (g_strcmp0 (search_type_id, PHOTOS_SEARCH_TYPE_STOCK_ALL) == 0
+      && g_strcmp0 (source_id, PHOTOS_SOURCE_STOCK_ALL) == 0
+      && (str == NULL || str [0] == '\0'))
+    mode = priv->old_mode;
+  else
+    mode = PHOTOS_WINDOW_MODE_SEARCH;
+
+  photos_mode_controller_set_window_mode (priv->mode_cntrlr, mode);
+
+  g_free (search_type_id);
+  g_free (source_id);
+}
+
+
 static void
 photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
                                   PhotosWindowMode mode,
@@ -414,12 +552,16 @@ photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
 
+  self->priv->old_mode = old_mode;
+
   if (mode == PHOTOS_WINDOW_MODE_COLLECTIONS)
     photos_embed_prepare_for_collections (self);
   else if (mode == PHOTOS_WINDOW_MODE_FAVORITES)
     photos_embed_prepare_for_favorites (self);
   else if (mode == PHOTOS_WINDOW_MODE_OVERVIEW)
     photos_embed_prepare_for_overview (self);
+  else if (mode == PHOTOS_WINDOW_MODE_SEARCH)
+    photos_embed_prepare_for_search (self);
   else
     photos_embed_prepare_for_preview (self);
 }
@@ -431,13 +573,17 @@ photos_embed_dispose (GObject *object)
   PhotosEmbed *self = PHOTOS_EMBED (object);
   PhotosEmbedPrivate *priv = self->priv;
 
+  photos_embed_clear_search (self);
+
   g_clear_object (&priv->ntfctn_mngr);
   g_clear_object (&priv->loader_cancellable);
   g_clear_object (&priv->indexing_ntfctn);
+  g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
+  g_clear_object (&priv->src_mngr);
+  g_clear_object (&priv->srch_mngr);
   g_clear_object (&priv->mode_cntrlr);
-  g_clear_object (&priv->offset_cntrlr);
-  g_clear_object (&priv->monitor);
+  g_clear_object (&priv->srch_cntrlr);
   g_clear_object (&priv->trk_ovrvw_cntrlr);
 
   /* GdStack triggers notify::visible-child during dispose and this means that
@@ -473,6 +619,7 @@ photos_embed_init (PhotosEmbed *self)
   priv = self->priv;
 
   app = photos_application_new ();
+  priv->search_action = g_action_map_lookup_action (G_ACTION_MAP (app), "search");
   g_signal_connect_swapped (app, "window-added", G_CALLBACK (photos_embed_window_added), self);
   g_object_unref (app);
 
@@ -492,7 +639,7 @@ photos_embed_init (PhotosEmbed *self)
   gtk_widget_show (priv->stack);
   gtk_container_add (GTK_CONTAINER (priv->stack_overlay), priv->stack);
 
-  priv->toolbar = photos_main_toolbar_new ();
+  priv->toolbar = photos_main_toolbar_new (GTK_OVERLAY (priv->stack_overlay));
   photos_main_toolbar_set_stack (PHOTOS_MAIN_TOOLBAR (priv->toolbar), GTK_STACK (priv->stack));
 
   priv->ntfctn_mngr = g_object_ref_sink (photos_notification_manager_dup_singleton ());
@@ -509,18 +656,15 @@ photos_embed_init (PhotosEmbed *self)
   priv->favorites = photos_view_container_new (PHOTOS_WINDOW_MODE_FAVORITES);
   gtk_stack_add_titled (GTK_STACK (priv->stack), priv->favorites, "favorites", _("Favorites"));
 
+  priv->search = photos_view_container_new (PHOTOS_WINDOW_MODE_SEARCH);
+  gtk_stack_add_named (GTK_STACK (priv->stack), priv->search, "search");
+
   priv->preview = photos_preview_view_new (GTK_OVERLAY (priv->stack_overlay));
   gtk_stack_add_named (GTK_STACK (priv->stack), priv->preview, "preview");
 
   priv->spinner_box = photos_spinner_box_new ();
   gtk_stack_add_named (GTK_STACK (priv->stack), priv->spinner_box, "spinner");
 
-  priv->error_box = photos_error_box_new ();
-  gtk_stack_add_named (GTK_STACK (priv->stack), priv->error_box, "error");
-
-  priv->no_results = photos_empty_results_box_new ();
-  gtk_stack_add_named (GTK_STACK (priv->stack), priv->no_results, "no-results");
-
   /* TODO: SearchBar.Dropdown, …
    */
 
@@ -529,6 +673,7 @@ photos_embed_init (PhotosEmbed *self)
                            self, G_CONNECT_SWAPPED);
 
   priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
+  priv->old_mode = PHOTOS_WINDOW_MODE_NONE;
   g_signal_connect (priv->mode_cntrlr,
                     "window-mode-changed",
                     G_CALLBACK (photos_embed_window_mode_changed),
@@ -539,22 +684,31 @@ photos_embed_init (PhotosEmbed *self)
                     self);
 
   priv->trk_ovrvw_cntrlr = photos_tracker_overview_controller_dup_singleton ();
-  g_signal_connect_swapped (priv->trk_ovrvw_cntrlr, "query-error", G_CALLBACK (photos_embed_query_error), self);
   g_signal_connect_swapped (priv->trk_ovrvw_cntrlr,
                             "query-status-changed",
                             G_CALLBACK (photos_embed_query_status_changed),
                             self);
 
-  priv->offset_cntrlr = photos_offset_overview_controller_dup_singleton ();
-  g_signal_connect_swapped (priv->offset_cntrlr, "count-changed", G_CALLBACK (photos_embed_count_changed), self);
+  priv->col_mngr = photos_collection_manager_dup_singleton ();
+  g_signal_connect (priv->col_mngr, "active-changed", G_CALLBACK (photos_embed_active_changed), self);
 
   priv->item_mngr = photos_item_manager_dup_singleton ();
   g_signal_connect (priv->item_mngr, "active-changed", G_CALLBACK (photos_embed_active_changed), self);
 
+  priv->src_mngr = photos_source_manager_dup_singleton ();
+  g_signal_connect_swapped (priv->src_mngr, "active-changed", G_CALLBACK (photos_embed_search_changed), self);
+
+  priv->srch_mngr = photos_search_type_manager_dup_singleton ();
+  g_signal_connect_swapped (priv->srch_mngr, "active-changed", G_CALLBACK (photos_embed_search_changed), self);
+
   querying = photos_tracker_controller_get_query_status (priv->trk_ovrvw_cntrlr);
   photos_embed_query_status_changed (self, querying);
 
-  priv->monitor = photos_tracker_change_monitor_dup_singleton (NULL, NULL);
+  priv->srch_cntrlr = photos_search_controller_dup_singleton ();
+  g_signal_connect_swapped (priv->srch_cntrlr,
+                            "search-string-changed",
+                            G_CALLBACK (photos_embed_search_changed),
+                            self);
 
   gtk_widget_show (GTK_WIDGET (self));
 }
@@ -574,3 +728,10 @@ photos_embed_new (void)
 {
   return g_object_new (PHOTOS_TYPE_EMBED, NULL);
 }
+
+
+PhotosMainToolbar *
+photos_embed_get_main_toolbar (PhotosEmbed *self)
+{
+  return PHOTOS_MAIN_TOOLBAR (self->priv->toolbar);
+}
