@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 #include <glib/gi18n.h>
 
 #include "photos-empty-results-box.h"
+#include "photos-enums.h"
+#include "photos-icons.h"
 #include "photos-source-manager.h"
 
 
@@ -35,6 +37,13 @@ struct _PhotosEmptyResultsBoxPrivate
 {
   GtkWidget *labels_grid;
   PhotosBaseManager *src_mngr;
+  PhotosWindowMode mode;
+};
+
+enum
+{
+  PROP_0,
+  PROP_MODE
 };
 
 
@@ -94,6 +103,21 @@ photos_empty_results_box_activate_link (PhotosEmptyResultsBox *self, const gchar
 }
 
 
+static void
+photos_empty_results_box_add_collections_label (PhotosEmptyResultsBox *self)
+{
+  GtkWidget *details;
+
+  details = gtk_label_new (_("Name your first album"));
+  gtk_widget_set_halign (details, GTK_ALIGN_START);
+  gtk_misc_set_alignment (GTK_MISC (details), 0.0, 0.5);
+  gtk_label_set_line_wrap (GTK_LABEL (details), TRUE);
+  gtk_label_set_max_width_chars (GTK_LABEL (details), 24);
+  gtk_label_set_use_markup (GTK_LABEL (details), TRUE);
+  gtk_container_add (GTK_CONTAINER (self->priv->labels_grid), details);
+}
+
+
 static void
 photos_empty_results_box_add_system_settings_label (PhotosEmptyResultsBox *self)
 {
@@ -102,13 +126,13 @@ photos_empty_results_box_add_system_settings_label (PhotosEmptyResultsBox *self)
   gchar *system_settings_href;
 
   /* Translators: this should be translated in the context of the "You
-   * can add your online accounts in System Settings" sentence below
+   * can add your online accounts in Settings" sentence below
    */
-  system_settings_href = g_strconcat ("<a href=\"system-settings\">", _("System Settings"), "</a>", NULL);
+  system_settings_href = g_strconcat ("<a href=\"system-settings\">", _("Settings"), "</a>", NULL);
 
-  /* Translators: %s here is "System Settings", which is in a separate
-   * string due to markup, and should be translated only in the context
-   * of this sentence.
+  /* Translators: %s here is "Settings", which is in a separate string
+   * due to markup, and should be translated only in the context of
+   * this sentence.
    */
   details_str = g_strdup_printf (_("You can add your online accounts in %s"), system_settings_href);
 
@@ -148,7 +172,7 @@ photos_empty_results_box_constructed (GObject *object)
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
   gtk_style_context_add_class (context, "dim-label");
 
-  image = gtk_image_new_from_icon_name ("emblem-photos-symbolic", GTK_ICON_SIZE_INVALID);
+  image = gtk_image_new_from_icon_name (PHOTOS_ICON_PHOTOS_SYMBOLIC, GTK_ICON_SIZE_INVALID);
   gtk_image_set_pixel_size (GTK_IMAGE (image), 64);
   gtk_container_add (GTK_CONTAINER (self), image);
 
@@ -157,7 +181,26 @@ photos_empty_results_box_constructed (GObject *object)
   gtk_grid_set_row_spacing (GTK_GRID (priv->labels_grid), 12);
   gtk_container_add (GTK_CONTAINER (self), priv->labels_grid);
 
-  label = g_strconcat ("<b><span size=\"large\">", _("No Photos Found"), "</span></b>", NULL);
+  switch (priv->mode)
+    {
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+      label = g_strconcat ("<b><span size=\"large\">", _("No Albums Found"), "</span></b>", NULL);
+      break;
+
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+      label = g_strconcat ("<b><span size=\"large\">", _("No Favorites Found"), "</span></b>", NULL);
+      break;
+
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      label = g_strconcat ("<b><span size=\"large\">", _("No Photos Found"), "</span></b>", NULL);
+      break;
+
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+
   title_label = gtk_label_new (label);
   gtk_widget_set_halign (title_label, GTK_ALIGN_START);
   gtk_widget_set_vexpand (title_label, TRUE);
@@ -165,12 +208,31 @@ photos_empty_results_box_constructed (GObject *object)
   gtk_container_add (GTK_CONTAINER (priv->labels_grid), title_label);
   g_free (label);
 
-  if (photos_source_manager_has_online_sources (PHOTOS_SOURCE_MANAGER (self->priv->src_mngr)))
-    gtk_widget_set_valign (title_label, GTK_ALIGN_CENTER);
-  else
+  switch (priv->mode)
     {
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
       gtk_widget_set_valign (title_label, GTK_ALIGN_START);
-      photos_empty_results_box_add_system_settings_label (self);
+      photos_empty_results_box_add_collections_label (self);
+      break;
+
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      gtk_widget_set_valign (title_label, GTK_ALIGN_CENTER);
+      break;
+
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+      if (photos_source_manager_has_online_sources (PHOTOS_SOURCE_MANAGER (self->priv->src_mngr)))
+        gtk_widget_set_valign (title_label, GTK_ALIGN_CENTER);
+      else
+        {
+          gtk_widget_set_valign (title_label, GTK_ALIGN_START);
+          photos_empty_results_box_add_system_settings_label (self);
+        }
+      break;
+
+    default:
+      g_assert_not_reached ();
+      break;
     }
 
   gtk_widget_show_all (GTK_WIDGET (self));
@@ -188,6 +250,24 @@ photos_empty_results_box_dispose (GObject *object)
 }
 
 
+static void
+photos_empty_results_box_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+  PhotosEmptyResultsBox *self = PHOTOS_EMPTY_RESULTS_BOX (object);
+
+  switch (prop_id)
+    {
+    case PROP_MODE:
+      self->priv->mode = (PhotosWindowMode) g_value_get_enum (value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
 static void
 photos_empty_results_box_init (PhotosEmptyResultsBox *self)
 {
@@ -207,11 +287,21 @@ photos_empty_results_box_class_init (PhotosEmptyResultsBoxClass *class)
 
   object_class->constructed = photos_empty_results_box_constructed;
   object_class->dispose = photos_empty_results_box_dispose;
+  object_class->set_property = photos_empty_results_box_set_property;
+
+  g_object_class_install_property (object_class,
+                                   PROP_MODE,
+                                   g_param_spec_enum ("mode",
+                                                      "PhotosWindowMode enum",
+                                                      "The mode for which no results were found",
+                                                      PHOTOS_TYPE_WINDOW_MODE,
+                                                      PHOTOS_WINDOW_MODE_NONE,
+                                                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
 }
 
 
 GtkWidget *
-photos_empty_results_box_new (void)
+photos_empty_results_box_new (PhotosWindowMode mode)
 {
-  return g_object_new (PHOTOS_TYPE_EMPTY_RESULTS_BOX, NULL);
+  return g_object_new (PHOTOS_TYPE_EMPTY_RESULTS_BOX, "mode", mode, NULL);
 }
