@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,10 +33,13 @@
 #include "photos-application.h"
 #include "photos-collection-manager.h"
 #include "photos-dlna-renderers-manager.h"
+#include "photos-dropdown.h"
 #include "photos-header-bar.h"
+#include "photos-icons.h"
 #include "photos-item-manager.h"
 #include "photos-main-toolbar.h"
 #include "photos-mode-controller.h"
+#include "photos-overview-searchbar.h"
 #include "photos-remote-display-manager.h"
 #include "photos-selection-controller.h"
 #include "photos-source-manager.h"
@@ -44,9 +47,12 @@
 
 struct _PhotosMainToolbarPrivate
 {
+  GAction *search;
   GSimpleAction *gear_menu;
   GtkWidget *coll_back_button;
+  GtkWidget *overlay;
   GtkWidget *remote_display_button;
+  GtkWidget *searchbar;
   GtkWidget *selection_menu;
   GtkWidget *toolbar;
   PhotosBaseManager *col_mngr;
@@ -58,6 +64,12 @@ struct _PhotosMainToolbarPrivate
   PhotosWindowMode old_mode;
 };
 
+enum
+{
+  PROP_0,
+  PROP_OVERLAY
+};
+
 
 G_DEFINE_TYPE_WITH_PRIVATE (PhotosMainToolbar, photos_main_toolbar, GTK_TYPE_BOX);
 
@@ -77,7 +89,8 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
 
   if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW
       || window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS
-      || window_mode == PHOTOS_WINDOW_MODE_FAVORITES)
+      || window_mode == PHOTOS_WINDOW_MODE_FAVORITES
+      || window_mode == PHOTOS_WINDOW_MODE_SEARCH)
     {
       if (!selection_mode)
         {
@@ -140,7 +153,7 @@ photos_main_toolbar_add_back_button (PhotosMainToolbar *self)
   const gchar *icon_name;
 
   direction = gtk_widget_get_direction (GTK_WIDGET (priv->toolbar));
-  icon_name = (direction == GTK_TEXT_DIR_RTL) ? "go-next-symbolic" : "go-previous-symbolic";
+  icon_name = (direction == GTK_TEXT_DIR_RTL) ? PHOTOS_ICON_GO_NEXT_SYMBOLIC : PHOTOS_ICON_GO_PREVIOUS_SYMBOLIC;
 
   back_button = gd_header_simple_button_new ();
   gd_header_button_set_label (GD_HEADER_BUTTON (back_button), _("Back"));
@@ -251,7 +264,7 @@ photos_main_toolbar_add_remote_display_button (PhotosMainToolbar *self)
   gd_header_button_set_use_markup (GD_HEADER_BUTTON (priv->remote_display_button), TRUE);
   label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (priv->remote_display_button)));
   gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_MIDDLE);
-  gtk_widget_set_margin_right (priv->remote_display_button, 12);
+  gtk_widget_set_margin_end (priv->remote_display_button, 12);
   gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->toolbar), priv->remote_display_button);
   gtk_widget_show_all (priv->remote_display_button);
 
@@ -261,6 +274,21 @@ photos_main_toolbar_add_remote_display_button (PhotosMainToolbar *self)
 }
 
 
+static GtkWidget *
+photos_main_toolbar_add_search_button (PhotosMainToolbar *self)
+{
+  GtkWidget *search_button;
+
+  search_button = gd_header_toggle_button_new ();
+  gd_header_button_set_label (GD_HEADER_BUTTON (search_button), _("Search"));
+  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (search_button), PHOTOS_ICON_EDIT_FIND_SYMBOLIC);
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (search_button), "app.search");
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (self->priv->toolbar), search_button);
+
+  return search_button;
+}
+
+
 static void
 photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
 {
@@ -269,7 +297,7 @@ photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
 
   selection_button = gd_header_simple_button_new ();
   gd_header_button_set_label (GD_HEADER_BUTTON (selection_button), _("Select Items"));
-  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (selection_button), "object-select-symbolic");
+  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (selection_button), PHOTOS_ICON_OBJECT_SELECT_SYMBOLIC);
   gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->toolbar), selection_button);
   g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_select_button_clicked), self);
 
@@ -309,6 +337,17 @@ photos_main_toolbar_clear_state_data (PhotosMainToolbar *self)
       priv->remote_display_button = NULL;
     }
 
+  if (priv->searchbar != NULL && gtk_widget_get_parent (priv->searchbar) == GTK_WIDGET (self))
+    {
+      GVariant *state;
+
+      state = g_action_get_state (priv->search);
+      if (!g_variant_get_boolean (state))
+        gtk_container_remove (GTK_CONTAINER (self), priv->searchbar);
+
+      g_variant_unref (state);
+    }
+
   if (priv->col_mngr != NULL)
     g_signal_handlers_disconnect_by_func (priv->col_mngr, photos_main_toolbar_col_active_changed, self);
 
@@ -332,6 +371,20 @@ photos_main_toolbar_clear_toolbar (PhotosMainToolbar *self)
 }
 
 
+static GtkWidget *
+photos_main_toolbar_create_overview_searchbar (PhotosMainToolbar *self)
+{
+  GtkWidget *dropdown;
+  GtkWidget *searchbar;
+
+  dropdown = photos_dropdown_new ();
+  gtk_overlay_add_overlay (GTK_OVERLAY (self->priv->overlay), dropdown);
+  searchbar = photos_overview_searchbar_new (PHOTOS_DROPDOWN (dropdown));
+
+  return searchbar;
+}
+
+
 static GMenu *
 photos_main_toolbar_create_preview_menu (PhotosMainToolbar *self)
 {
@@ -385,6 +438,10 @@ photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
+  photos_main_toolbar_add_search_button (self);
+
+  if (gtk_widget_get_parent (priv->searchbar) == NULL)
+    gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
   photos_main_toolbar_col_active_changed (priv->col_mngr, object, self);
@@ -418,6 +475,10 @@ photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
+  photos_main_toolbar_add_search_button (self);
+
+  if (gtk_widget_get_parent (priv->searchbar) == NULL)
+    gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
   photos_main_toolbar_col_active_changed (priv->col_mngr, object, self);
@@ -433,6 +494,10 @@ photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
+  photos_main_toolbar_add_search_button (self);
+
+  if (gtk_widget_get_parent (priv->searchbar) == NULL)
+    gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
   photos_main_toolbar_col_active_changed (priv->col_mngr, object, self);
@@ -460,7 +525,7 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   menu_button = gd_header_menu_button_new ();
   gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_button), "app.gear-menu");
   gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (menu_button), G_MENU_MODEL (preview_menu));
-  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (menu_button), "emblem-system-symbolic");
+  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (menu_button), PHOTOS_ICON_SYSTEM_SYMBOLIC);
   gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->toolbar), menu_button);
 
   g_simple_action_set_enabled (priv->gear_menu, TRUE);
@@ -477,6 +542,25 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
 }
 
 
+static void
+photos_main_toolbar_populate_for_search (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  GObject *object;
+
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
+  photos_main_toolbar_add_selection_button (self);
+  photos_main_toolbar_add_search_button (self);
+
+  if (gtk_widget_get_parent (priv->searchbar) == NULL)
+    gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
+
+  object = photos_base_manager_get_active_object (priv->col_mngr);
+  photos_main_toolbar_col_active_changed (priv->col_mngr, object, self);
+}
+
+
 static void
 photos_main_toolbar_populate_for_selection_mode (PhotosMainToolbar *self)
 {
@@ -495,6 +579,8 @@ photos_main_toolbar_populate_for_selection_mode (PhotosMainToolbar *self)
                            G_CALLBACK (photos_main_toolbar_set_toolbar_title),
                            self,
                            G_CONNECT_SWAPPED);
+
+  photos_main_toolbar_add_search_button (self);
 }
 
 
@@ -519,6 +605,8 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
     photos_main_toolbar_populate_for_overview (self);
   else if (window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
     photos_main_toolbar_populate_for_preview (self);
+  else if (window_mode == PHOTOS_WINDOW_MODE_SEARCH)
+    photos_main_toolbar_populate_for_search (self);
 
   photos_main_toolbar_update_remote_display_button (self);
 
@@ -535,6 +623,21 @@ photos_main_toolbar_window_mode_changed (PhotosMainToolbar *self, PhotosWindowMo
 }
 
 
+static void
+photos_main_toolbar_constructed (GObject *object)
+{
+  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (object);
+  PhotosMainToolbarPrivate *priv = self->priv;
+
+  G_OBJECT_CLASS (photos_main_toolbar_parent_class)->constructed (object);
+
+  priv->searchbar = photos_main_toolbar_create_overview_searchbar (self);
+  g_object_ref (priv->searchbar);
+
+  photos_main_toolbar_reset_toolbar_mode (self);
+}
+
+
 static void
 photos_main_toolbar_dispose (GObject *object)
 {
@@ -543,6 +646,7 @@ photos_main_toolbar_dispose (GObject *object)
 
   photos_main_toolbar_clear_state_data (self);
 
+  g_clear_object (&priv->searchbar);
   g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
@@ -554,6 +658,24 @@ photos_main_toolbar_dispose (GObject *object)
 }
 
 
+static void
+photos_main_toolbar_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (object);
+
+  switch (prop_id)
+    {
+    case PROP_OVERLAY:
+      self->priv->overlay = GTK_WIDGET (g_value_dup_object (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
 static void
 photos_main_toolbar_share_changed_cb (PhotosMainToolbar          *self,
                                       PhotosDlnaRenderer         *renderer,
@@ -597,6 +719,7 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
 
   app = photos_application_new ();
   priv->gear_menu = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app), "gear-menu"));
+  priv->search = g_action_map_lookup_action (G_ACTION_MAP (app), "search");
   g_object_unref (app);
 
   builder = gtk_builder_new ();
@@ -652,8 +775,6 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
                            G_CALLBACK (photos_main_toolbar_share_error_cb),
                            self,
                            G_CONNECT_SWAPPED);
-
-  photos_main_toolbar_reset_toolbar_mode (self);
 }
 
 
@@ -662,14 +783,41 @@ photos_main_toolbar_class_init (PhotosMainToolbarClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
+  object_class->constructed = photos_main_toolbar_constructed;
   object_class->dispose = photos_main_toolbar_dispose;
+  object_class->set_property = photos_main_toolbar_set_property;
+
+  g_object_class_install_property (object_class,
+                                   PROP_OVERLAY,
+                                   g_param_spec_object ("overlay",
+                                                        "GtkOverlay object",
+                                                        "The stack overlay widget",
+                                                        GTK_TYPE_OVERLAY,
+                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
 }
 
 
 GtkWidget *
-photos_main_toolbar_new (void)
+photos_main_toolbar_new (GtkOverlay *overlay)
 {
-  return g_object_new (PHOTOS_TYPE_MAIN_TOOLBAR, NULL);
+  return g_object_new (PHOTOS_TYPE_MAIN_TOOLBAR, "overlay", overlay, NULL);
+}
+
+
+gboolean
+photos_main_toolbar_handle_event (PhotosMainToolbar *self, GdkEventKey *event)
+{
+  gboolean ret_val = FALSE;
+
+  ret_val = photos_searchbar_handle_event (PHOTOS_SEARCHBAR (self->priv->searchbar), event);
+  return ret_val;
+}
+
+
+gboolean
+photos_main_toolbar_is_focus (PhotosMainToolbar *self)
+{
+  return photos_searchbar_is_focus (PHOTOS_SEARCHBAR (self->priv->searchbar));
 }
 
 
