@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,10 +29,12 @@
 #include <glib/gi18n.h>
 
 #include "photos-about-data.h"
+#include "photos-collection-manager.h"
 #include "photos-embed.h"
 #include "photos-item-manager.h"
 #include "photos-main-window.h"
 #include "photos-mode-controller.h"
+#include "photos-selection-controller.h"
 #include "photos-settings.h"
 
 
@@ -40,8 +42,11 @@ struct _PhotosMainWindowPrivate
 {
   GtkWidget *embed;
   GSettings *settings;
+  PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
-  PhotosModeController *controller;
+  PhotosModeController *mode_cntrlr;
+  PhotosSelectionController *sel_cntrlr;
+  PhotosWindowMode old_mode;
   guint configure_id;
 };
 
@@ -103,7 +108,7 @@ photos_main_window_configure_event (GtkWidget *widget, GdkEventConfigure *event)
 
   ret_val = GTK_WIDGET_CLASS (photos_main_window_parent_class)->configure_event (widget, event);
 
-  if (photos_mode_controller_get_fullscreen (priv->controller))
+  if (photos_mode_controller_get_fullscreen (priv->mode_cntrlr))
     return ret_val;
 
   if (priv->configure_id != 0)
@@ -135,10 +140,8 @@ photos_main_window_delete_event (GtkWidget *widget, GdkEventAny *event)
 
 
 static void
-photos_main_window_fullscreen_changed (PhotosModeController *controller, gboolean fullscreen, gpointer user_data)
+photos_main_window_fullscreen_changed (PhotosMainWindow *self, gboolean fullscreen)
 {
-  PhotosMainWindow *self = PHOTOS_MAIN_WINDOW (user_data);
-
   if (fullscreen)
     gtk_window_fullscreen (GTK_WINDOW (self));
   else
@@ -146,10 +149,79 @@ photos_main_window_fullscreen_changed (PhotosModeController *controller, gboolea
 }
 
 
+static gboolean
+photos_main_window_go_back (PhotosMainWindow *self)
+{
+  PhotosMainWindowPrivate *priv = self->priv;
+  PhotosWindowMode mode;
+  GObject *active_collection;
+  gboolean handled = TRUE;
+
+  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  active_collection = photos_base_manager_get_active_object (priv->col_mngr);
+
+  switch (mode)
+    {
+    case PHOTOS_WINDOW_MODE_PREVIEW:
+      photos_base_manager_set_active_object (priv->item_mngr, NULL);
+      photos_mode_controller_set_window_mode (priv->mode_cntrlr, priv->old_mode);
+      break;
+
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      if (active_collection != NULL)
+        photos_item_manager_activate_previous_collection (PHOTOS_ITEM_MANAGER (self->priv->item_mngr));
+      break;
+
+    default:
+      handled = FALSE;
+      break;
+    }
+
+  return handled;
+}
+
+
+static gboolean
+photos_main_window_is_back_key (PhotosMainWindow *self, GdkEventKey *event)
+{
+  GtkTextDirection direction;
+  gboolean is_back;
+
+  direction = gtk_widget_get_direction (GTK_WIDGET (self));
+  is_back = (((event->state & GDK_MOD1_MASK) != 0
+              && ((direction == GTK_TEXT_DIR_LTR && event->keyval == GDK_KEY_Left)
+                  || (direction == GTK_TEXT_DIR_RTL && event->keyval == GDK_KEY_Right)))
+             || event->keyval == GDK_KEY_Back);
+
+  return is_back;
+}
+
+
+static gboolean
+photos_main_window_handle_back_key (PhotosMainWindow *self, GdkEventKey *event)
+{
+  if (!photos_main_window_is_back_key (self, event))
+    return FALSE;
+
+  return photos_main_window_go_back (self);
+}
+
+
 static gboolean
 photos_main_window_handle_key_overview (PhotosMainWindow *self, GdkEventKey *event)
 {
-  return GDK_EVENT_PROPAGATE;
+  PhotosMainWindowPrivate *priv = self->priv;
+  gboolean handled = FALSE;
+
+  if (photos_selection_controller_get_selection_mode (priv->sel_cntrlr) && event->keyval == GDK_KEY_Escape)
+    {
+      photos_selection_controller_set_selection_mode (priv->sel_cntrlr, FALSE);
+      handled = TRUE;
+    }
+
+  return handled;
 }
 
 
@@ -157,21 +229,16 @@ static gboolean
 photos_main_window_handle_key_preview (PhotosMainWindow *self, GdkEventKey *event)
 {
   PhotosMainWindowPrivate *priv = self->priv;
-  GtkTextDirection direction;
   gboolean fullscreen;
 
-  direction = gtk_widget_get_direction (GTK_WIDGET (self));
-  fullscreen = photos_mode_controller_get_fullscreen (priv->controller);
-
-  if ((fullscreen && event->keyval == GDK_KEY_Escape)
-      || ((event->state & GDK_MOD1_MASK) != 0
-          && ((direction == GTK_TEXT_DIR_LTR && event->keyval == GDK_KEY_Left)
-              || (direction == GTK_TEXT_DIR_RTL && event->keyval == GDK_KEY_Right)))
-      || event->keyval == GDK_KEY_BackSpace
-      || event->keyval == GDK_KEY_Back)
+  fullscreen = photos_mode_controller_get_fullscreen (priv->mode_cntrlr);
+  if (event->keyval == GDK_KEY_Escape)
     {
-      photos_base_manager_set_active_object (priv->item_mngr, NULL);
-      return GDK_EVENT_STOP;
+      if (fullscreen)
+        {
+          photos_base_manager_set_active_object (priv->item_mngr, NULL);
+          photos_mode_controller_set_window_mode (priv->mode_cntrlr, priv->old_mode);
+        }
     }
 
   return GDK_EVENT_PROPAGATE;
@@ -183,15 +250,40 @@ photos_main_window_key_press_event (GtkWidget *widget, GdkEventKey *event)
 {
   PhotosMainWindow *self = PHOTOS_MAIN_WINDOW (widget);
   PhotosMainWindowPrivate *priv = self->priv;
+  PhotosMainToolbar *toolbar;
   PhotosWindowMode mode;
-  gboolean handled;
+  gboolean handled = GDK_EVENT_PROPAGATE;
 
-  mode = photos_mode_controller_get_window_mode (priv->controller);
-  if (mode == PHOTOS_WINDOW_MODE_PREVIEW)
-    handled = photos_main_window_handle_key_preview (self, event);
-  else
-    handled = photos_main_window_handle_key_overview (self, event);
+  toolbar = photos_embed_get_main_toolbar (PHOTOS_EMBED (priv->embed));
+  handled = photos_main_toolbar_handle_event (toolbar, event);
+  if (handled)
+    goto out;
 
+  handled = photos_main_window_handle_back_key (self, event);
+  if (handled)
+    goto out;
+
+  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+
+  switch (mode)
+    {
+    case PHOTOS_WINDOW_MODE_NONE:
+      handled = GDK_EVENT_PROPAGATE;
+      break;
+
+    case PHOTOS_WINDOW_MODE_PREVIEW:
+      handled = photos_main_window_handle_key_preview (self, event);
+      break;
+
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      handled = photos_main_window_handle_key_overview (self, event);
+      break;
+    }
+
+ out:
   if (!handled)
     handled = GTK_WIDGET_CLASS (photos_main_window_parent_class)->key_press_event (widget, event);
 
@@ -199,6 +291,13 @@ photos_main_window_key_press_event (GtkWidget *widget, GdkEventKey *event)
 }
 
 
+static void
+photos_main_window_window_mode_changed (PhotosMainWindow *self, PhotosWindowMode mode, PhotosWindowMode old_mode)
+{
+  self->priv->old_mode = old_mode;
+}
+
+
 static gboolean
 photos_main_window_window_state_event (GtkWidget *widget, GdkEventWindowState *event)
 {
@@ -230,8 +329,10 @@ photos_main_window_dispose (GObject *object)
   PhotosMainWindowPrivate *priv = self->priv;
 
   g_clear_object (&priv->settings);
+  g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
-  g_clear_object (&priv->controller);
+  g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->sel_cntrlr);
 
   if (priv->configure_id != 0)
     {
@@ -274,13 +375,22 @@ photos_main_window_init (PhotosMainWindow *self)
   if (maximized)
     gtk_window_maximize (GTK_WINDOW (self));
 
+  priv->col_mngr = photos_collection_manager_dup_singleton ();
   priv->item_mngr = photos_item_manager_dup_singleton ();
 
-  priv->controller = photos_mode_controller_dup_singleton ();
-  g_signal_connect (priv->controller,
-                    "fullscreen-changed",
-                    G_CALLBACK (photos_main_window_fullscreen_changed),
-                    self);
+  priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
+  priv->old_mode = PHOTOS_WINDOW_MODE_NONE;
+  g_signal_connect_swapped (priv->mode_cntrlr,
+                            "fullscreen-changed",
+                            G_CALLBACK (photos_main_window_fullscreen_changed),
+                            self);
+  g_signal_connect_object (priv->mode_cntrlr,
+                           "window-mode-changed",
+                           G_CALLBACK (photos_main_window_window_mode_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  priv->sel_cntrlr = photos_selection_controller_dup_singleton ();
 
   priv->embed = photos_embed_new ();
   gtk_container_add (GTK_CONTAINER (self), priv->embed);
@@ -329,7 +439,9 @@ photos_main_window_show_about (PhotosMainWindow *self)
   gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about_dialog), PHOTOS_AUTHORS);
   gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about_dialog),
                                  _("Access, organize and share your photos on GNOME"));
-  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (about_dialog), "Copyright © 2012, 2013 Red Hat, Inc.");
+  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (about_dialog),
+                                  "Copyright © 2013 Intel Corporation. All rights reserved.\n"
+                                  "Copyright © 2012, 2013, 2014 Red Hat, Inc.");
   gtk_about_dialog_set_license_type (GTK_ABOUT_DIALOG (about_dialog), GTK_LICENSE_GPL_2_0);
   gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (about_dialog), PACKAGE_TARNAME);
   gtk_about_dialog_set_program_name (GTK_ABOUT_DIALOG (about_dialog), _(PACKAGE_NAME));
