@@ -119,7 +119,7 @@ photos_offset_controller_init (PhotosOffsetController *self)
   self->priv = photos_offset_controller_get_instance_private (self);
   priv = self->priv;
 
-  priv->queue = photos_tracker_queue_dup_singleton ();
+  priv->queue = photos_tracker_queue_dup_singleton (NULL, NULL);
 }
 
 
@@ -208,6 +208,9 @@ photos_offset_controller_reset_count (PhotosOffsetController *self)
   PhotosOffsetControllerPrivate *priv = self->priv;
   PhotosQuery *query;
 
+  if (G_UNLIKELY (priv->queue == NULL))
+    return;
+
   query = PHOTOS_OFFSET_CONTROLLER_GET_CLASS (self)->get_query (self);
   photos_tracker_queue_select (priv->queue,
                                query->sparql,
