@@ -63,7 +63,8 @@ typedef enum
   PHOTOS_QUERY_FLAGS_UNFILTERED = 1 << 0,
   PHOTOS_QUERY_FLAGS_COLLECTIONS = 1 << 1,
   PHOTOS_QUERY_FLAGS_FAVORITES = 1 << 2,
-  PHOTOS_QUERY_FLAGS_OVERVIEW = 1 << 3
+  PHOTOS_QUERY_FLAGS_OVERVIEW = 1 << 3,
+  PHOTOS_QUERY_FLAGS_SEARCH = 1 << 4
 } PhotosQueryFlags;
 
 extern const gchar *PHOTOS_QUERY_COLLECTIONS_IDENTIFIER;
