@@ -37,6 +37,55 @@
 G_DEFINE_TYPE (PhotosSearchTypeManager, photos_search_type_manager, PHOTOS_TYPE_BASE_MANAGER);
 
 
+static gchar *
+photos_search_type_manager_get_filter (PhotosBaseManager *mngr, gint flags)
+{
+  GObject *search_type;
+  gchar *filter;
+  gchar *id;
+
+  if (flags & PHOTOS_QUERY_FLAGS_COLLECTIONS)
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_COLLECTIONS);
+  else if (flags & PHOTOS_QUERY_FLAGS_FAVORITES)
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_FAVORITES);
+  else if (flags & PHOTOS_QUERY_FLAGS_OVERVIEW)
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_PHOTOS);
+  else if (flags & PHOTOS_QUERY_FLAGS_SEARCH)
+    search_type = photos_base_manager_get_active_object (mngr);
+  else
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_ALL);
+
+  g_object_get (search_type, "id", &id, NULL);
+  if (g_strcmp0 (id, PHOTOS_SEARCH_TYPE_STOCK_ALL) == 0)
+    filter = photos_base_manager_get_all_filter (mngr);
+  else
+    filter = photos_filterable_get_filter (PHOTOS_FILTERABLE (search_type));
+
+  g_free (id);
+  return filter;
+}
+
+
+static gchar *
+photos_search_type_manager_get_where (PhotosBaseManager *mngr, gint flags)
+{
+  GObject *search_type;
+
+  if (flags & PHOTOS_QUERY_FLAGS_COLLECTIONS)
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_COLLECTIONS);
+  else if (flags & PHOTOS_QUERY_FLAGS_FAVORITES)
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_FAVORITES);
+  else if (flags & PHOTOS_QUERY_FLAGS_OVERVIEW)
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_PHOTOS);
+  else if (flags & PHOTOS_QUERY_FLAGS_SEARCH)
+    search_type = photos_base_manager_get_active_object (mngr);
+  else
+    search_type = photos_base_manager_get_object_by_id (mngr, PHOTOS_SEARCH_TYPE_STOCK_ALL);
+
+  return photos_filterable_get_where (PHOTOS_FILTERABLE (search_type));
+}
+
+
 static GObject *
 photos_search_type_manager_constructor (GType                  type,
                                         guint                  n_construct_params,
@@ -74,7 +123,7 @@ photos_search_type_manager_init (PhotosSearchTypeManager *self)
   search_type = photos_search_type_new_full (PHOTOS_SEARCH_TYPE_STOCK_ALL,
                                              _("All"),
                                              "?urn a rdfs:Resource",
-                                             "(false)");
+                                             "(false)"); /* unused */
   photos_base_manager_add_object (PHOTOS_BASE_MANAGER (self), G_OBJECT (search_type));
   g_object_unref (search_type);
 
@@ -110,23 +159,19 @@ static void
 photos_search_type_manager_class_init (PhotosSearchTypeManagerClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
+  PhotosBaseManagerClass *base_manager_class = PHOTOS_BASE_MANAGER_CLASS (class);
 
   object_class->constructor = photos_search_type_manager_constructor;
+  base_manager_class->get_filter = photos_search_type_manager_get_filter;
+  base_manager_class->get_where = photos_search_type_manager_get_where;
 }
 
 
 PhotosBaseManager *
 photos_search_type_manager_dup_singleton (void)
 {
-  return g_object_new (PHOTOS_TYPE_SEARCH_TYPE_MANAGER, NULL);
-}
-
-
-gchar *
-photos_search_type_manager_get_where (PhotosSearchTypeManager *self)
-{
-  GObject *search_type;
-
-  search_type = photos_base_manager_get_active_object (PHOTOS_BASE_MANAGER (self));
-  return photos_filterable_get_where (PHOTOS_FILTERABLE (search_type));
+  /* Translators: "Type" refers to a search filter. eg., All, Albums,
+   * Favorites and Photos.
+   */
+  return g_object_new (PHOTOS_TYPE_SEARCH_TYPE_MANAGER, "title", C_("Search Filter", "Type"), NULL);
 }
