@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +31,7 @@
 
 #include "photos-application.h"
 #include "photos-base-item.h"
+#include "photos-icons.h"
 #include "photos-item-manager.h"
 #include "photos-organize-collection-dialog.h"
 #include "photos-properties-dialog.h"
@@ -53,6 +54,7 @@ struct _PhotosSelectionToolbarPrivate
   gboolean inside_refresh;
 };
 
+
 G_DEFINE_TYPE_WITH_PRIVATE (PhotosSelectionToolbar, photos_selection_toolbar, GTK_TYPE_REVEALER);
 
 
@@ -67,7 +69,7 @@ photos_selection_toolbar_dialog_response (GtkDialog *dialog, gint response_id, g
 {
   PhotosSelectionToolbar *self = PHOTOS_SELECTION_TOOLBAR (user_data);
 
-  if (response_id != GTK_RESPONSE_OK)
+  if (response_id != GTK_RESPONSE_CLOSE)
     return;
 
   gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -426,7 +428,8 @@ photos_selection_toolbar_init (PhotosSelectionToolbar *self)
   gtk_container_add (GTK_CONTAINER (self), toolbar);
 
   priv->toolbar_favorite = gd_header_toggle_button_new ();
-  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (priv->toolbar_favorite), "emblem-favorite-symbolic");
+  gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (priv->toolbar_favorite),
+                                           PHOTOS_ICON_FAVORITE_SYMBOLIC);
   gtk_action_bar_pack_start (GTK_ACTION_BAR (toolbar), priv->toolbar_favorite);
   g_signal_connect (priv->toolbar_favorite,
                     "clicked",
