@@ -28,7 +28,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include "photos-collection-manager.h"
 #include "photos-enums.h"
 #include "photos-item-manager.h"
 #include "photos-marshalers.h"
@@ -43,7 +42,6 @@ struct _PhotosTrackerControllerPrivate
 {
   GCancellable *cancellable;
   GError *queue_error;
-  PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
   PhotosBaseManager *src_mngr;
   PhotosModeController *mode_cntrlr;
@@ -207,13 +205,6 @@ photos_tracker_controller_offset_changed (PhotosTrackerController *self)
 }
 
 
-static void
-photos_tracker_controller_refresh_for_object (PhotosTrackerController *self)
-{
-  photos_tracker_controller_refresh_internal (self, PHOTOS_TRACKER_REFRESH_FLAGS_RESET_OFFSET);
-}
-
-
 static void
 photos_tracker_controller_set_query_status (PhotosTrackerController *self, gboolean query_status)
 {
@@ -256,7 +247,6 @@ photos_tracker_controller_refresh_internal (PhotosTrackerController *self, gint
     }
 
   photos_tracker_controller_set_query_status (self, TRUE);
-  photos_base_manager_clear (priv->item_mngr);
   photos_tracker_controller_perform_current_query (self);
 }
 
@@ -336,7 +326,6 @@ photos_tracker_controller_dispose (GObject *object)
   PhotosTrackerController *self = PHOTOS_TRACKER_CONTROLLER (object);
   PhotosTrackerControllerPrivate *priv = self->priv;
 
-  g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
   g_clear_object (&priv->mode_cntrlr);
@@ -391,12 +380,6 @@ photos_tracker_controller_init (PhotosTrackerController *self)
   priv->cancellable = g_cancellable_new ();
   priv->item_mngr = photos_item_manager_dup_singleton ();
 
-  priv->col_mngr = photos_collection_manager_dup_singleton ();
-  g_signal_connect_swapped (priv->col_mngr,
-                            "active-changed",
-                            G_CALLBACK (photos_tracker_controller_refresh_for_object),
-                            self);
-
   priv->src_mngr = photos_source_manager_dup_singleton ();
   g_signal_connect_swapped (priv->src_mngr,
                             "object-added",
@@ -406,10 +389,6 @@ photos_tracker_controller_init (PhotosTrackerController *self)
                             "object-removed",
                             G_CALLBACK (photos_tracker_controller_source_object_added_removed),
                             self);
-  g_signal_connect_swapped (priv->src_mngr,
-                            "active-changed",
-                            G_CALLBACK (photos_tracker_controller_refresh_for_object),
-                            self);
 
   priv->queue = photos_tracker_queue_dup_singleton (NULL, &priv->queue_error);
 }
@@ -461,6 +440,13 @@ photos_tracker_controller_class_init (PhotosTrackerControllerClass *class)
 }
 
 
+void
+photos_tracker_controller_refresh_for_object (PhotosTrackerController *self)
+{
+  photos_tracker_controller_refresh_internal (self, PHOTOS_TRACKER_REFRESH_FLAGS_RESET_OFFSET);
+}
+
+
 void
 photos_tracker_controller_start (PhotosTrackerController *self)
 {
