@@ -124,7 +124,7 @@ photos_update_mtime_job_init (PhotosUpdateMtimeJob *self)
   self->priv = photos_update_mtime_job_get_instance_private (self);
   priv = self->priv;
 
-  priv->queue = photos_tracker_queue_dup_singleton ();
+  priv->queue = photos_tracker_queue_dup_singleton (NULL, NULL);
 }
 
 
@@ -162,6 +162,13 @@ photos_update_mtime_job_run (PhotosUpdateMtimeJob *self,
   PhotosUpdateMtimeJobPrivate *priv = self->priv;
   PhotosQuery *query;
 
+  if (G_UNLIKELY (priv->queue == NULL))
+    {
+      if (callback != NULL)
+        (*callback) (user_data);
+      return;
+    }
+
   priv->callback = callback;
   priv->user_data = user_data;
 
