@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,14 +27,21 @@
 #include <libgd/gd.h>
 
 #include "photos-application.h"
+#include "photos-empty-results-box.h"
 #include "photos-enums.h"
+#include "photos-error-box.h"
 #include "photos-item-manager.h"
 #include "photos-load-more-button.h"
+#include "photos-offset-favorites-controller.h"
+#include "photos-offset-collections-controller.h"
+#include "photos-offset-overview-controller.h"
+#include "photos-offset-search-controller.h"
 #include "photos-remote-display-manager.h"
 #include "photos-selection-controller.h"
 #include "photos-tracker-collections-controller.h"
 #include "photos-tracker-favorites-controller.h"
 #include "photos-tracker-overview-controller.h"
+#include "photos-tracker-search-controller.h"
 #include "photos-utils.h"
 #include "photos-view-container.h"
 #include "photos-view-model.h"
@@ -45,9 +52,12 @@ struct _PhotosViewContainerPrivate
   GdMainView *view;
   GtkListStore *model;
   GtkTreePath *current_path;
+  GtkWidget *error_box;
   GtkWidget *load_more;
+  GtkWidget *no_results;
   PhotosBaseManager *item_mngr;
   PhotosModeController *mode_cntrlr;
+  PhotosOffsetController *offset_cntrlr;
   PhotosRemoteDisplayManager *remote_mngr;
   PhotosSelectionController *sel_cntrlr;
   PhotosTrackerController *trk_cntrlr;
@@ -62,7 +72,7 @@ enum
 };
 
 
-G_DEFINE_TYPE_WITH_PRIVATE (PhotosViewContainer, photos_view_container, GTK_TYPE_GRID);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosViewContainer, photos_view_container, GTK_TYPE_STACK);
 
 
 static void
@@ -129,6 +139,16 @@ photos_view_container_connect_view (PhotosViewContainer *self)
 }
 
 
+static void
+photos_view_container_count_changed (PhotosViewContainer *self, gint count)
+{
+  if (count == 0)
+    gtk_stack_set_visible_child_name (GTK_STACK (self), "no-results");
+  else
+    gtk_stack_set_visible_child_name (GTK_STACK (self), "view");
+}
+
+
 static void
 photos_view_container_disconnect_view (PhotosViewContainer *self)
 {
@@ -165,6 +185,23 @@ photos_view_container_item_activated (GdMainView *main_view,
 }
 
 
+static void
+photos_view_container_set_error (PhotosViewContainer *self, const gchar *primary, const gchar *secondary)
+{
+  PhotosViewContainerPrivate *priv = self->priv;
+
+  photos_error_box_update (PHOTOS_ERROR_BOX (priv->error_box), primary, secondary);
+  gtk_stack_set_visible_child_name (GTK_STACK (self), "error");
+}
+
+
+static void
+photos_view_container_query_error (PhotosViewContainer *self, const gchar *primary, const gchar *secondary)
+{
+  photos_view_container_set_error (self, primary, secondary);
+}
+
+
 static void
 photos_view_container_query_status_changed (PhotosTrackerController *trk_cntrlr,
                                             gboolean query_status,
@@ -234,7 +271,8 @@ photos_view_container_window_mode_changed (PhotosModeController *mode_cntrlr,
 
   if (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
       || mode == PHOTOS_WINDOW_MODE_FAVORITES
-      || mode == PHOTOS_WINDOW_MODE_OVERVIEW)
+      || mode == PHOTOS_WINDOW_MODE_OVERVIEW
+      || mode == PHOTOS_WINDOW_MODE_SEARCH)
     photos_view_container_connect_view (self);
 }
 
@@ -246,22 +284,33 @@ photos_view_container_constructed (GObject *object)
   PhotosViewContainerPrivate *priv = self->priv;
   GAction *action;
   GtkApplication *app;
+  GtkWidget *grid;
   gboolean status;
 
   G_OBJECT_CLASS (photos_view_container_parent_class)->constructed (object);
 
   priv->model = photos_view_model_new (priv->mode);
 
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
+  grid = gtk_grid_new ();
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
+  gtk_stack_add_named (GTK_STACK (self), grid, "view");
+
+  priv->no_results = photos_empty_results_box_new (priv->mode);
+  gtk_stack_add_named (GTK_STACK (self), priv->no_results, "no-results");
+
+  priv->error_box = photos_error_box_new ();
+  gtk_stack_add_named (GTK_STACK (self), priv->error_box, "error");
 
   priv->view = gd_main_view_new (GD_MAIN_VIEW_ICON);
-  gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (priv->view));
+  gtk_container_add (GTK_CONTAINER (grid), GTK_WIDGET (priv->view));
 
   priv->load_more = photos_load_more_button_new (priv->mode);
-  gtk_container_add (GTK_CONTAINER (self), priv->load_more);
+  gtk_container_add (GTK_CONTAINER (grid), priv->load_more);
 
   gtk_widget_show_all (GTK_WIDGET (self));
 
+  gtk_stack_set_visible_child_full (GTK_STACK (self), "view", GTK_STACK_TRANSITION_TYPE_NONE);
+
   g_signal_connect (priv->view, "item-activated", G_CALLBACK (photos_view_container_item_activated), self);
   g_signal_connect (priv->view,
                     "selection-mode-request",
@@ -295,14 +344,22 @@ photos_view_container_constructed (GObject *object)
     {
     case PHOTOS_WINDOW_MODE_COLLECTIONS:
       priv->trk_cntrlr = photos_tracker_collections_controller_dup_singleton ();
+      priv->offset_cntrlr = photos_offset_collections_controller_dup_singleton ();
       break;
 
     case PHOTOS_WINDOW_MODE_FAVORITES:
       priv->trk_cntrlr = photos_tracker_favorites_controller_dup_singleton ();
+      priv->offset_cntrlr = photos_offset_favorites_controller_dup_singleton ();
       break;
 
     case PHOTOS_WINDOW_MODE_OVERVIEW:
       priv->trk_cntrlr = photos_tracker_overview_controller_dup_singleton ();
+      priv->offset_cntrlr = photos_offset_overview_controller_dup_singleton ();
+      break;
+
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      priv->trk_cntrlr = photos_tracker_search_controller_dup_singleton ();
+      priv->offset_cntrlr = photos_offset_search_controller_dup_singleton ();
       break;
 
     default:
@@ -320,6 +377,17 @@ photos_view_container_constructed (GObject *object)
 
   g_object_unref (app);
 
+  g_signal_connect_object (priv->offset_cntrlr,
+                           "count-changed",
+                           G_CALLBACK (photos_view_container_count_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  g_signal_connect_object (priv->trk_cntrlr,
+                           "query-error",
+                           G_CALLBACK (photos_view_container_query_error),
+                           self,
+                           G_CONNECT_SWAPPED);
   g_signal_connect (priv->trk_cntrlr,
                     "query-status-changed",
                     G_CALLBACK (photos_view_container_query_status_changed),
@@ -346,6 +414,7 @@ photos_view_container_dispose (GObject *object)
   g_clear_object (&priv->model);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->offset_cntrlr);
   g_clear_object (&priv->remote_mngr);
   g_clear_object (&priv->sel_cntrlr);
   g_clear_object (&priv->trk_cntrlr);
@@ -415,7 +484,11 @@ photos_view_container_class_init (PhotosViewContainerClass *class)
 GtkWidget *
 photos_view_container_new (PhotosWindowMode mode)
 {
-  return g_object_new (PHOTOS_TYPE_VIEW_CONTAINER, "mode", mode, NULL);
+  return g_object_new (PHOTOS_TYPE_VIEW_CONTAINER,
+                       "homogeneous", TRUE,
+                       "mode", mode,
+                       "transition-type", GTK_STACK_TRANSITION_TYPE_CROSSFADE,
+                       NULL);
 }
 
 
