@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,15 @@
 #include "photos-collection-manager.h"
 #include "photos-enums.h"
 #include "photos-item-manager.h"
+#include "photos-mode-controller.h"
+#include "photos-offset-collections-controller.h"
+#include "photos-offset-favorites-controller.h"
+#include "photos-offset-overview-controller.h"
+#include "photos-offset-search-controller.h"
+#include "photos-tracker-collections-controller.h"
+#include "photos-tracker-favorites-controller.h"
+#include "photos-tracker-overview-controller.h"
+#include "photos-tracker-search-controller.h"
 #include "photos-view-model.h"
 
 
@@ -35,8 +44,14 @@ struct _PhotosViewModelPrivate
 {
   PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
+  PhotosModeController *mode_cntrlr;
+  PhotosOffsetController *offset_cntrlr;
+  PhotosTrackerController *trk_cntrlr;
   PhotosWindowMode mode;
   gchar *row_ref_key;
+  gint n_rows;
+  gint64 oldest_mtime;
+  guint reset_count_id;
 };
 
 enum
@@ -49,6 +64,12 @@ enum
 G_DEFINE_TYPE_WITH_PRIVATE (PhotosViewModel, photos_view_model, GTK_TYPE_LIST_STORE);
 
 
+enum
+{
+  RESET_COUNT_TIMEOUT = 500 /* ms */
+};
+
+
 static void
 photos_view_model_info_set (PhotosViewModel *self, PhotosBaseItem *item, GtkTreeIter *iter)
 {
@@ -64,16 +85,53 @@ photos_view_model_info_set (PhotosViewModel *self, PhotosBaseItem *item, GtkTree
 }
 
 
+static gboolean
+photos_view_model_reset_count_timeout (gpointer user_data)
+{
+  PhotosViewModel *self = PHOTOS_VIEW_MODEL (user_data);
+  PhotosViewModelPrivate *priv = self->priv;
+
+  priv->reset_count_id = 0;
+  photos_offset_controller_reset_count (priv->offset_cntrlr);
+  return G_SOURCE_REMOVE;
+}
+
+
 static void
 photos_view_model_add_item (PhotosViewModel *self, PhotosBaseItem *item)
 {
+  PhotosViewModelPrivate *priv = self->priv;
   GtkTreeIter iter;
   GtkTreePath *path;
   GtkTreeRowReference *row_ref;
+  gint offset;
+  gint step;
+  gint64 mtime;
+
+  /* Update the count so that PhotosOffsetController has the correct
+   * values. Otherwise things like the "Load More" button will not
+   * work correctly.
+   */
+  if (priv->reset_count_id == 0)
+    priv->reset_count_id = g_timeout_add_full (G_PRIORITY_DEFAULT,
+                                               RESET_COUNT_TIMEOUT,
+                                               photos_view_model_reset_count_timeout,
+                                               g_object_ref (self),
+                                               g_object_unref);
+
+  offset = photos_offset_controller_get_offset (priv->offset_cntrlr);
+  step = photos_offset_controller_get_step (priv->offset_cntrlr);
+  mtime = photos_base_item_get_mtime (item);
+  if (priv->n_rows >= offset + step && mtime < priv->oldest_mtime)
+    return;
 
   gtk_list_store_append (GTK_LIST_STORE (self), &iter);
   photos_view_model_info_set (self, item, &iter);
 
+  priv->n_rows++;
+  if (mtime < priv->oldest_mtime)
+    priv->oldest_mtime = mtime;
+
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (self), &iter);
   row_ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (self), path);
   gtk_tree_path_free (path);
@@ -85,25 +143,57 @@ photos_view_model_add_item (PhotosViewModel *self, PhotosBaseItem *item)
 }
 
 
+static void
+photos_view_model_clear (PhotosViewModel *self)
+{
+  PhotosViewModelPrivate *priv = self->priv;
+  GHashTable *items;
+  GHashTableIter iter;
+  PhotosBaseItem *item;
+
+  items = photos_base_manager_get_objects (priv->item_mngr);
+  g_hash_table_iter_init (&iter, items);
+  while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &item))
+    g_object_set_data (G_OBJECT (item), priv->row_ref_key, NULL);
+
+  gtk_list_store_clear (GTK_LIST_STORE (self));
+
+  priv->n_rows = 0;
+  priv->oldest_mtime = G_MAXINT64;
+}
+
+
 static gboolean
 photos_view_model_item_removed_foreach (GtkTreeModel *model,
                                         GtkTreePath *path,
                                         GtkTreeIter *iter,
                                         gpointer user_data)
 {
+  PhotosViewModel *self = PHOTOS_VIEW_MODEL (model);
+  PhotosViewModelPrivate *priv = self->priv;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (user_data);
   gboolean ret_val = FALSE;
   const gchar *id;
   gchar *value;
+  gint64 mtime;
 
   id = photos_base_item_get_id (item);
-  gtk_tree_model_get (model, iter, PHOTOS_VIEW_MODEL_URN, &value, -1);
+  gtk_tree_model_get (model, iter, PHOTOS_VIEW_MODEL_URN, &value, PHOTOS_VIEW_MODEL_MTIME, &mtime, -1);
 
   if (g_strcmp0 (id, value) == 0)
     {
+      GtkTreeIter tmp;
+
+      tmp = *iter;
+      if (!gtk_tree_model_iter_next (model, &tmp))
+        ret_val = TRUE;
+
       gtk_list_store_remove (GTK_LIST_STORE (model), iter);
-      ret_val = TRUE;
+      gtk_tree_path_next (path); /* Ensure that path in sync with iter. */
+      priv->n_rows--;
     }
+  else if (mtime < priv->oldest_mtime)
+    priv->oldest_mtime = mtime;
 
   g_free (value);
   return ret_val;
@@ -113,10 +203,12 @@ photos_view_model_item_removed_foreach (GtkTreeModel *model,
 static void
 photos_view_model_object_removed (PhotosViewModel *self, GObject *object)
 {
+  PhotosViewModelPrivate *priv = self->priv;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (object);
 
+  priv->oldest_mtime = G_MAXINT64;
   gtk_tree_model_foreach (GTK_TREE_MODEL (self), photos_view_model_item_removed_foreach, item);
-  g_object_set_data (object, self->priv->row_ref_key, NULL);
+  g_object_set_data (object, priv->row_ref_key, NULL);
 }
 
 
@@ -125,12 +217,12 @@ photos_view_model_info_updated (PhotosBaseItem *item, gpointer user_data)
 {
   PhotosViewModel *self = PHOTOS_VIEW_MODEL (user_data);
   PhotosViewModelPrivate *priv = self->priv;
-  GObject *collection;
+  GObject *active_collection;
   GtkTreeIter iter;
   GtkTreePath *path;
   GtkTreeRowReference *row_ref;
 
-  collection = photos_base_manager_get_active_object (priv->col_mngr);
+  active_collection = photos_base_manager_get_active_object (priv->col_mngr);
   row_ref = (GtkTreeRowReference *) g_object_get_data (G_OBJECT (item), priv->row_ref_key);
 
   if (priv->mode == PHOTOS_WINDOW_MODE_COLLECTIONS)
@@ -138,9 +230,9 @@ photos_view_model_info_updated (PhotosBaseItem *item, gpointer user_data)
       gboolean is_collection;
 
       is_collection = photos_base_item_is_collection (item);
-      if (!is_collection && row_ref != NULL && collection == NULL)
+      if (!is_collection && row_ref != NULL && active_collection == NULL)
         photos_view_model_object_removed (self, G_OBJECT (item));
-      else if (is_collection  && row_ref == NULL)
+      else if (is_collection  && row_ref == NULL && active_collection == NULL)
         photos_view_model_add_item (self, item);
     }
   else if (priv->mode == PHOTOS_WINDOW_MODE_FAVORITES)
@@ -148,9 +240,9 @@ photos_view_model_info_updated (PhotosBaseItem *item, gpointer user_data)
       gboolean is_favorite;
 
       is_favorite = photos_base_item_is_favorite (item);
-      if (!is_favorite && row_ref != NULL && collection == NULL)
+      if (!is_favorite && row_ref != NULL && active_collection == NULL)
         photos_view_model_object_removed (self, G_OBJECT (item));
-      else if (is_favorite  && row_ref == NULL && collection == NULL)
+      else if (is_favorite  && row_ref == NULL && active_collection == NULL)
         photos_view_model_add_item (self, item);
     }
   else if (priv->mode == PHOTOS_WINDOW_MODE_OVERVIEW)
@@ -182,15 +274,22 @@ photos_view_model_object_added (PhotosViewModel *self, GObject *object)
 {
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (object);
   PhotosViewModelPrivate *priv = self->priv;
-  GObject *collection;
+  PhotosWindowMode mode;
+  GObject *active_collection;
+  GtkTreeRowReference *row_ref;
   gboolean is_collection;
   gboolean is_favorite;
 
-  collection = photos_base_manager_get_active_object (priv->col_mngr);
+  row_ref = (GtkTreeRowReference *) g_object_get_data (G_OBJECT (item), priv->row_ref_key);
+  if (row_ref != NULL)
+    return;
+
+  active_collection = photos_base_manager_get_active_object (priv->col_mngr);
   is_collection = photos_base_item_is_collection (item);
   is_favorite = photos_base_item_is_favorite (item);
+  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
 
-  if (collection == NULL)
+  if (active_collection == NULL || priv->mode != mode)
     {
       if ((priv->mode == PHOTOS_WINDOW_MODE_COLLECTIONS && !is_collection)
           || (priv->mode == PHOTOS_WINDOW_MODE_FAVORITES && !is_favorite)
@@ -205,14 +304,78 @@ photos_view_model_object_added (PhotosViewModel *self, GObject *object)
 }
 
 
+static void
+photos_view_model_query_status_changed (PhotosViewModel *self, gboolean query_status)
+{
+  if (query_status == FALSE)
+    return;
+
+  photos_view_model_clear (self);
+}
+
+
+static void
+photos_view_model_constructed (GObject *object)
+{
+  PhotosViewModel *self = PHOTOS_VIEW_MODEL (object);
+  PhotosViewModelPrivate *priv = self->priv;
+
+  G_OBJECT_CLASS (photos_view_model_parent_class)->constructed (object);
+
+  switch (priv->mode)
+    {
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+      priv->offset_cntrlr = photos_offset_collections_controller_dup_singleton ();
+      priv->trk_cntrlr = photos_tracker_collections_controller_dup_singleton ();
+      break;
+
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+      priv->offset_cntrlr = photos_offset_favorites_controller_dup_singleton ();
+      priv->trk_cntrlr = photos_tracker_favorites_controller_dup_singleton ();
+      break;
+
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+      priv->offset_cntrlr = photos_offset_overview_controller_dup_singleton ();
+      priv->trk_cntrlr = photos_tracker_overview_controller_dup_singleton ();
+      break;
+
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      priv->offset_cntrlr = photos_offset_search_controller_dup_singleton ();
+      priv->trk_cntrlr = photos_tracker_search_controller_dup_singleton ();
+      break;
+
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+
+  g_signal_connect_swapped (priv->item_mngr, "object-added", G_CALLBACK (photos_view_model_object_added), self);
+  g_signal_connect_swapped (priv->item_mngr, "object-removed", G_CALLBACK (photos_view_model_object_removed), self);
+
+  g_signal_connect_swapped (priv->trk_cntrlr,
+                            "query-status-changed",
+                            G_CALLBACK (photos_view_model_query_status_changed),
+                            self);
+}
+
+
 static void
 photos_view_model_dispose (GObject *object)
 {
   PhotosViewModel *self = PHOTOS_VIEW_MODEL (object);
   PhotosViewModelPrivate *priv = self->priv;
 
+  if (priv->reset_count_id != 0)
+    {
+      g_source_remove (priv->reset_count_id);
+      priv->reset_count_id = 0;
+    }
+
   g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
+  g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->offset_cntrlr);
+  g_clear_object (&priv->trk_cntrlr);
 
   G_OBJECT_CLASS (photos_view_model_parent_class)->dispose (object);
 }
@@ -259,7 +422,8 @@ photos_view_model_init (PhotosViewModel *self)
                      G_TYPE_STRING,    /* AUTHOR */
                      GDK_TYPE_PIXBUF,  /* ICON */
                      G_TYPE_INT64,     /* MTIME */
-                     G_TYPE_BOOLEAN};  /* STATE */
+                     G_TYPE_BOOLEAN,   /* STATE */
+                     G_TYPE_UINT};     /* PULSE (unused) */
 
   self->priv = photos_view_model_get_instance_private (self);
   priv = self->priv;
@@ -268,11 +432,10 @@ photos_view_model_init (PhotosViewModel *self)
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self), PHOTOS_VIEW_MODEL_MTIME, GTK_SORT_DESCENDING);
 
   priv->col_mngr = photos_collection_manager_dup_singleton ();
-
   priv->item_mngr = photos_item_manager_dup_singleton ();
-  g_signal_connect_swapped (priv->item_mngr, "object-added", G_CALLBACK (photos_view_model_object_added), self);
-  g_signal_connect_swapped (priv->item_mngr, "object-removed", G_CALLBACK (photos_view_model_object_removed), self);
-  g_signal_connect_swapped (priv->item_mngr, "clear", G_CALLBACK (gtk_list_store_clear), self);
+  priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
+
+  priv->oldest_mtime = G_MAXINT64;
 }
 
 
@@ -281,6 +444,7 @@ photos_view_model_class_init (PhotosViewModelClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
+  object_class->constructed = photos_view_model_constructed;
   object_class->dispose = photos_view_model_dispose;
   object_class->finalize = photos_view_model_finalize;
   object_class->set_property = photos_view_model_set_property;
